/*******************************************************************************
* File Name: Pulse_signal_Flow.c  
* Version 2.20
*
* Description:
*  This file contains APIs to set up the Pins component for low power modes.
*
* Note:
*
********************************************************************************
* Copyright 2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "cytypes.h"
#include "Pulse_signal_Flow.h"

static Pulse_signal_Flow_BACKUP_STRUCT  Pulse_signal_Flow_backup = {0u, 0u, 0u};


/*******************************************************************************
* Function Name: Pulse_signal_Flow_Sleep
****************************************************************************//**
*
* \brief Stores the pin configuration and prepares the pin for entering chip 
*  deep-sleep/hibernate modes. This function applies only to SIO and USBIO pins.
*  It should not be called for GPIO or GPIO_OVT pins.
*
* <b>Note</b> This function is available in PSoC 4 only.
*
* \return 
*  None 
*  
* \sideeffect
*  For SIO pins, this function configures the pin input threshold to CMOS and
*  drive level to Vddio. This is needed for SIO pins when in device 
*  deep-sleep/hibernate modes.
*
* \funcusage
*  \snippet Pulse_signal_Flow_SUT.c usage_Pulse_signal_Flow_Sleep_Wakeup
*******************************************************************************/
void Pulse_signal_Flow_Sleep(void)
{
    #if defined(Pulse_signal_Flow__PC)
        Pulse_signal_Flow_backup.pcState = Pulse_signal_Flow_PC;
    #else
        #if (CY_PSOC4_4200L)
            /* Save the regulator state and put the PHY into suspend mode */
            Pulse_signal_Flow_backup.usbState = Pulse_signal_Flow_CR1_REG;
            Pulse_signal_Flow_USB_POWER_REG |= Pulse_signal_Flow_USBIO_ENTER_SLEEP;
            Pulse_signal_Flow_CR1_REG &= Pulse_signal_Flow_USBIO_CR1_OFF;
        #endif
    #endif
    #if defined(CYIPBLOCK_m0s8ioss_VERSION) && defined(Pulse_signal_Flow__SIO)
        Pulse_signal_Flow_backup.sioState = Pulse_signal_Flow_SIO_REG;
        /* SIO requires unregulated output buffer and single ended input buffer */
        Pulse_signal_Flow_SIO_REG &= (uint32)(~Pulse_signal_Flow_SIO_LPM_MASK);
    #endif  
}


/*******************************************************************************
* Function Name: Pulse_signal_Flow_Wakeup
****************************************************************************//**
*
* \brief Restores the pin configuration that was saved during Pin_Sleep(). This 
* function applies only to SIO and USBIO pins. It should not be called for
* GPIO or GPIO_OVT pins.
*
* For USBIO pins, the wakeup is only triggered for falling edge interrupts.
*
* <b>Note</b> This function is available in PSoC 4 only.
*
* \return 
*  None
*  
* \funcusage
*  Refer to Pulse_signal_Flow_Sleep() for an example usage.
*******************************************************************************/
void Pulse_signal_Flow_Wakeup(void)
{
    #if defined(Pulse_signal_Flow__PC)
        Pulse_signal_Flow_PC = Pulse_signal_Flow_backup.pcState;
    #else
        #if (CY_PSOC4_4200L)
            /* Restore the regulator state and come out of suspend mode */
            Pulse_signal_Flow_USB_POWER_REG &= Pulse_signal_Flow_USBIO_EXIT_SLEEP_PH1;
            Pulse_signal_Flow_CR1_REG = Pulse_signal_Flow_backup.usbState;
            Pulse_signal_Flow_USB_POWER_REG &= Pulse_signal_Flow_USBIO_EXIT_SLEEP_PH2;
        #endif
    #endif
    #if defined(CYIPBLOCK_m0s8ioss_VERSION) && defined(Pulse_signal_Flow__SIO)
        Pulse_signal_Flow_SIO_REG = Pulse_signal_Flow_backup.sioState;
    #endif
}


/* [] END OF FILE */
