/***************************************************************************//**
* \file SPI_Card.c
* \version 4.0
*
* \brief
*  This file provides the source code to the API for the SCB Component.
*
* Note:
*
*******************************************************************************
* \copyright
* Copyright 2013-2017, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "SPI_Card_PVT.h"

#if (SPI_Card_SCB_MODE_I2C_INC)
    #include "SPI_Card_I2C_PVT.h"
#endif /* (SPI_Card_SCB_MODE_I2C_INC) */

#if (SPI_Card_SCB_MODE_EZI2C_INC)
    #include "SPI_Card_EZI2C_PVT.h"
#endif /* (SPI_Card_SCB_MODE_EZI2C_INC) */

#if (SPI_Card_SCB_MODE_SPI_INC || SPI_Card_SCB_MODE_UART_INC)
    #include "SPI_Card_SPI_UART_PVT.h"
#endif /* (SPI_Card_SCB_MODE_SPI_INC || SPI_Card_SCB_MODE_UART_INC) */


/***************************************
*    Run Time Configuration Vars
***************************************/

/* Stores internal component configuration for Unconfigured mode */
#if (SPI_Card_SCB_MODE_UNCONFIG_CONST_CFG)
    /* Common configuration variables */
    uint8 SPI_Card_scbMode = SPI_Card_SCB_MODE_UNCONFIG;
    uint8 SPI_Card_scbEnableWake;
    uint8 SPI_Card_scbEnableIntr;

    /* I2C configuration variables */
    uint8 SPI_Card_mode;
    uint8 SPI_Card_acceptAddr;

    /* SPI/UART configuration variables */
    volatile uint8 * SPI_Card_rxBuffer;
    uint8  SPI_Card_rxDataBits;
    uint32 SPI_Card_rxBufferSize;

    volatile uint8 * SPI_Card_txBuffer;
    uint8  SPI_Card_txDataBits;
    uint32 SPI_Card_txBufferSize;

    /* EZI2C configuration variables */
    uint8 SPI_Card_numberOfAddr;
    uint8 SPI_Card_subAddrSize;
#endif /* (SPI_Card_SCB_MODE_UNCONFIG_CONST_CFG) */


/***************************************
*     Common SCB Vars
***************************************/
/**
* \addtogroup group_general
* \{
*/

/** SPI_Card_initVar indicates whether the SPI_Card 
*  component has been initialized. The variable is initialized to 0 
*  and set to 1 the first time SCB_Start() is called. This allows 
*  the component to restart without reinitialization after the first 
*  call to the SPI_Card_Start() routine.
*
*  If re-initialization of the component is required, then the 
*  SPI_Card_Init() function can be called before the 
*  SPI_Card_Start() or SPI_Card_Enable() function.
*/
uint8 SPI_Card_initVar = 0u;


#if (! (SPI_Card_SCB_MODE_I2C_CONST_CFG || \
        SPI_Card_SCB_MODE_EZI2C_CONST_CFG))
    /** This global variable stores TX interrupt sources after 
    * SPI_Card_Stop() is called. Only these TX interrupt sources 
    * will be restored on a subsequent SPI_Card_Enable() call.
    */
    uint16 SPI_Card_IntrTxMask = 0u;
#endif /* (! (SPI_Card_SCB_MODE_I2C_CONST_CFG || \
              SPI_Card_SCB_MODE_EZI2C_CONST_CFG)) */
/** \} globals */

#if (SPI_Card_SCB_IRQ_INTERNAL)
#if !defined (CY_REMOVE_SPI_Card_CUSTOM_INTR_HANDLER)
    void (*SPI_Card_customIntrHandler)(void) = NULL;
#endif /* !defined (CY_REMOVE_SPI_Card_CUSTOM_INTR_HANDLER) */
#endif /* (SPI_Card_SCB_IRQ_INTERNAL) */


/***************************************
*    Private Function Prototypes
***************************************/

static void SPI_Card_ScbEnableIntr(void);
static void SPI_Card_ScbModeStop(void);
static void SPI_Card_ScbModePostEnable(void);


/*******************************************************************************
* Function Name: SPI_Card_Init
****************************************************************************//**
*
*  Initializes the SPI_Card component to operate in one of the selected
*  configurations: I2C, SPI, UART or EZI2C.
*  When the configuration is set to "Unconfigured SCB", this function does
*  not do any initialization. Use mode-specific initialization APIs instead:
*  SPI_Card_I2CInit, SPI_Card_SpiInit, 
*  SPI_Card_UartInit or SPI_Card_EzI2CInit.
*
*******************************************************************************/
void SPI_Card_Init(void)
{
#if (SPI_Card_SCB_MODE_UNCONFIG_CONST_CFG)
    if (SPI_Card_SCB_MODE_UNCONFIG_RUNTM_CFG)
    {
        SPI_Card_initVar = 0u;
    }
    else
    {
        /* Initialization was done before this function call */
    }

#elif (SPI_Card_SCB_MODE_I2C_CONST_CFG)
    SPI_Card_I2CInit();

#elif (SPI_Card_SCB_MODE_SPI_CONST_CFG)
    SPI_Card_SpiInit();

#elif (SPI_Card_SCB_MODE_UART_CONST_CFG)
    SPI_Card_UartInit();

#elif (SPI_Card_SCB_MODE_EZI2C_CONST_CFG)
    SPI_Card_EzI2CInit();

#endif /* (SPI_Card_SCB_MODE_UNCONFIG_CONST_CFG) */
}


/*******************************************************************************
* Function Name: SPI_Card_Enable
****************************************************************************//**
*
*  Enables SPI_Card component operation: activates the hardware and 
*  internal interrupt. It also restores TX interrupt sources disabled after the 
*  SPI_Card_Stop() function was called (note that level-triggered TX 
*  interrupt sources remain disabled to not cause code lock-up).
*  For I2C and EZI2C modes the interrupt is internal and mandatory for 
*  operation. For SPI and UART modes the interrupt can be configured as none, 
*  internal or external.
*  The SPI_Card configuration should be not changed when the component
*  is enabled. Any configuration changes should be made after disabling the 
*  component.
*  When configuration is set to “Unconfigured SPI_Card”, the component 
*  must first be initialized to operate in one of the following configurations: 
*  I2C, SPI, UART or EZ I2C, using the mode-specific initialization API. 
*  Otherwise this function does not enable the component.
*
*******************************************************************************/
void SPI_Card_Enable(void)
{
#if (SPI_Card_SCB_MODE_UNCONFIG_CONST_CFG)
    /* Enable SCB block, only if it is already configured */
    if (!SPI_Card_SCB_MODE_UNCONFIG_RUNTM_CFG)
    {
        SPI_Card_CTRL_REG |= SPI_Card_CTRL_ENABLED;

        SPI_Card_ScbEnableIntr();

        /* Call PostEnable function specific to current operation mode */
        SPI_Card_ScbModePostEnable();
    }
#else
    SPI_Card_CTRL_REG |= SPI_Card_CTRL_ENABLED;

    SPI_Card_ScbEnableIntr();

    /* Call PostEnable function specific to current operation mode */
    SPI_Card_ScbModePostEnable();
#endif /* (SPI_Card_SCB_MODE_UNCONFIG_CONST_CFG) */
}


/*******************************************************************************
* Function Name: SPI_Card_Start
****************************************************************************//**
*
*  Invokes SPI_Card_Init() and SPI_Card_Enable().
*  After this function call, the component is enabled and ready for operation.
*  When configuration is set to "Unconfigured SCB", the component must first be
*  initialized to operate in one of the following configurations: I2C, SPI, UART
*  or EZI2C. Otherwise this function does not enable the component.
*
* \globalvars
*  SPI_Card_initVar - used to check initial configuration, modified
*  on first function call.
*
*******************************************************************************/
void SPI_Card_Start(void)
{
    if (0u == SPI_Card_initVar)
    {
        SPI_Card_Init();
        SPI_Card_initVar = 1u; /* Component was initialized */
    }

    SPI_Card_Enable();
}


/*******************************************************************************
* Function Name: SPI_Card_Stop
****************************************************************************//**
*
*  Disables the SPI_Card component: disable the hardware and internal 
*  interrupt. It also disables all TX interrupt sources so as not to cause an 
*  unexpected interrupt trigger because after the component is enabled, the 
*  TX FIFO is empty.
*  Refer to the function SPI_Card_Enable() for the interrupt 
*  configuration details.
*  This function disables the SCB component without checking to see if 
*  communication is in progress. Before calling this function it may be 
*  necessary to check the status of communication to make sure communication 
*  is complete. If this is not done then communication could be stopped mid 
*  byte and corrupted data could result.
*
*******************************************************************************/
void SPI_Card_Stop(void)
{
#if (SPI_Card_SCB_IRQ_INTERNAL)
    SPI_Card_DisableInt();
#endif /* (SPI_Card_SCB_IRQ_INTERNAL) */

    /* Call Stop function specific to current operation mode */
    SPI_Card_ScbModeStop();

    /* Disable SCB IP */
    SPI_Card_CTRL_REG &= (uint32) ~SPI_Card_CTRL_ENABLED;

    /* Disable all TX interrupt sources so as not to cause an unexpected
    * interrupt trigger after the component will be enabled because the 
    * TX FIFO is empty.
    * For SCB IP v0, it is critical as it does not mask-out interrupt
    * sources when it is disabled. This can cause a code lock-up in the
    * interrupt handler because TX FIFO cannot be loaded after the block
    * is disabled.
    */
    SPI_Card_SetTxInterruptMode(SPI_Card_NO_INTR_SOURCES);

#if (SPI_Card_SCB_IRQ_INTERNAL)
    SPI_Card_ClearPendingInt();
#endif /* (SPI_Card_SCB_IRQ_INTERNAL) */
}


/*******************************************************************************
* Function Name: SPI_Card_SetRxFifoLevel
****************************************************************************//**
*
*  Sets level in the RX FIFO to generate a RX level interrupt.
*  When the RX FIFO has more entries than the RX FIFO level an RX level
*  interrupt request is generated.
*
*  \param level: Level in the RX FIFO to generate RX level interrupt.
*   The range of valid level values is between 0 and RX FIFO depth - 1.
*
*******************************************************************************/
void SPI_Card_SetRxFifoLevel(uint32 level)
{
    uint32 rxFifoCtrl;

    rxFifoCtrl = SPI_Card_RX_FIFO_CTRL_REG;

    rxFifoCtrl &= ((uint32) ~SPI_Card_RX_FIFO_CTRL_TRIGGER_LEVEL_MASK); /* Clear level mask bits */
    rxFifoCtrl |= ((uint32) (SPI_Card_RX_FIFO_CTRL_TRIGGER_LEVEL_MASK & level));

    SPI_Card_RX_FIFO_CTRL_REG = rxFifoCtrl;
}


/*******************************************************************************
* Function Name: SPI_Card_SetTxFifoLevel
****************************************************************************//**
*
*  Sets level in the TX FIFO to generate a TX level interrupt.
*  When the TX FIFO has less entries than the TX FIFO level an TX level
*  interrupt request is generated.
*
*  \param level: Level in the TX FIFO to generate TX level interrupt.
*   The range of valid level values is between 0 and TX FIFO depth - 1.
*
*******************************************************************************/
void SPI_Card_SetTxFifoLevel(uint32 level)
{
    uint32 txFifoCtrl;

    txFifoCtrl = SPI_Card_TX_FIFO_CTRL_REG;

    txFifoCtrl &= ((uint32) ~SPI_Card_TX_FIFO_CTRL_TRIGGER_LEVEL_MASK); /* Clear level mask bits */
    txFifoCtrl |= ((uint32) (SPI_Card_TX_FIFO_CTRL_TRIGGER_LEVEL_MASK & level));

    SPI_Card_TX_FIFO_CTRL_REG = txFifoCtrl;
}


#if (SPI_Card_SCB_IRQ_INTERNAL)
    /*******************************************************************************
    * Function Name: SPI_Card_SetCustomInterruptHandler
    ****************************************************************************//**
    *
    *  Registers a function to be called by the internal interrupt handler.
    *  First the function that is registered is called, then the internal interrupt
    *  handler performs any operation such as software buffer management functions
    *  before the interrupt returns.  It is the user's responsibility not to break
    *  the software buffer operations. Only one custom handler is supported, which
    *  is the function provided by the most recent call.
    *  At the initialization time no custom handler is registered.
    *
    *  \param func: Pointer to the function to register.
    *        The value NULL indicates to remove the current custom interrupt
    *        handler.
    *
    *******************************************************************************/
    void SPI_Card_SetCustomInterruptHandler(void (*func)(void))
    {
    #if !defined (CY_REMOVE_SPI_Card_CUSTOM_INTR_HANDLER)
        SPI_Card_customIntrHandler = func; /* Register interrupt handler */
    #else
        if (NULL != func)
        {
            /* Suppress compiler warning */
        }
    #endif /* !defined (CY_REMOVE_SPI_Card_CUSTOM_INTR_HANDLER) */
    }
#endif /* (SPI_Card_SCB_IRQ_INTERNAL) */


/*******************************************************************************
* Function Name: SPI_Card_ScbModeEnableIntr
****************************************************************************//**
*
*  Enables an interrupt for a specific mode.
*
*******************************************************************************/
static void SPI_Card_ScbEnableIntr(void)
{
#if (SPI_Card_SCB_IRQ_INTERNAL)
    /* Enable interrupt in NVIC */
    #if (SPI_Card_SCB_MODE_UNCONFIG_CONST_CFG)
        if (0u != SPI_Card_scbEnableIntr)
        {
            SPI_Card_EnableInt();
        }

    #else
        SPI_Card_EnableInt();

    #endif /* (SPI_Card_SCB_MODE_UNCONFIG_CONST_CFG) */
#endif /* (SPI_Card_SCB_IRQ_INTERNAL) */
}


/*******************************************************************************
* Function Name: SPI_Card_ScbModePostEnable
****************************************************************************//**
*
*  Calls the PostEnable function for a specific operation mode.
*
*******************************************************************************/
static void SPI_Card_ScbModePostEnable(void)
{
#if (SPI_Card_SCB_MODE_UNCONFIG_CONST_CFG)
#if (!SPI_Card_CY_SCBIP_V1)
    if (SPI_Card_SCB_MODE_SPI_RUNTM_CFG)
    {
        SPI_Card_SpiPostEnable();
    }
    else if (SPI_Card_SCB_MODE_UART_RUNTM_CFG)
    {
        SPI_Card_UartPostEnable();
    }
    else
    {
        /* Unknown mode: do nothing */
    }
#endif /* (!SPI_Card_CY_SCBIP_V1) */

#elif (SPI_Card_SCB_MODE_SPI_CONST_CFG)
    SPI_Card_SpiPostEnable();

#elif (SPI_Card_SCB_MODE_UART_CONST_CFG)
    SPI_Card_UartPostEnable();

#else
    /* Unknown mode: do nothing */
#endif /* (SPI_Card_SCB_MODE_UNCONFIG_CONST_CFG) */
}


/*******************************************************************************
* Function Name: SPI_Card_ScbModeStop
****************************************************************************//**
*
*  Calls the Stop function for a specific operation mode.
*
*******************************************************************************/
static void SPI_Card_ScbModeStop(void)
{
#if (SPI_Card_SCB_MODE_UNCONFIG_CONST_CFG)
    if (SPI_Card_SCB_MODE_I2C_RUNTM_CFG)
    {
        SPI_Card_I2CStop();
    }
    else if (SPI_Card_SCB_MODE_EZI2C_RUNTM_CFG)
    {
        SPI_Card_EzI2CStop();
    }
#if (!SPI_Card_CY_SCBIP_V1)
    else if (SPI_Card_SCB_MODE_SPI_RUNTM_CFG)
    {
        SPI_Card_SpiStop();
    }
    else if (SPI_Card_SCB_MODE_UART_RUNTM_CFG)
    {
        SPI_Card_UartStop();
    }
#endif /* (!SPI_Card_CY_SCBIP_V1) */
    else
    {
        /* Unknown mode: do nothing */
    }
#elif (SPI_Card_SCB_MODE_I2C_CONST_CFG)
    SPI_Card_I2CStop();

#elif (SPI_Card_SCB_MODE_EZI2C_CONST_CFG)
    SPI_Card_EzI2CStop();

#elif (SPI_Card_SCB_MODE_SPI_CONST_CFG)
    SPI_Card_SpiStop();

#elif (SPI_Card_SCB_MODE_UART_CONST_CFG)
    SPI_Card_UartStop();

#else
    /* Unknown mode: do nothing */
#endif /* (SPI_Card_SCB_MODE_UNCONFIG_CONST_CFG) */
}


#if (SPI_Card_SCB_MODE_UNCONFIG_CONST_CFG)
    /*******************************************************************************
    * Function Name: SPI_Card_SetPins
    ****************************************************************************//**
    *
    *  Sets the pins settings accordingly to the selected operation mode.
    *  Only available in the Unconfigured operation mode. The mode specific
    *  initialization function calls it.
    *  Pins configuration is set by PSoC Creator when a specific mode of operation
    *  is selected in design time.
    *
    *  \param mode:      Mode of SCB operation.
    *  \param subMode:   Sub-mode of SCB operation. It is only required for SPI and UART
    *             modes.
    *  \param uartEnableMask: enables TX or RX direction and RTS and CTS signals.
    *
    *******************************************************************************/
    void SPI_Card_SetPins(uint32 mode, uint32 subMode, uint32 uartEnableMask)
    {
        uint32 pinsDm[SPI_Card_SCB_PINS_NUMBER];
        uint32 i;
        
    #if (!SPI_Card_CY_SCBIP_V1)
        uint32 pinsInBuf = 0u;
    #endif /* (!SPI_Card_CY_SCBIP_V1) */
        
        uint32 hsiomSel[SPI_Card_SCB_PINS_NUMBER] = 
        {
            SPI_Card_RX_SCL_MOSI_HSIOM_SEL_GPIO,
            SPI_Card_TX_SDA_MISO_HSIOM_SEL_GPIO,
            0u,
            0u,
            0u,
            0u,
            0u,
        };

    #if (SPI_Card_CY_SCBIP_V1)
        /* Supress compiler warning. */
        if ((0u == subMode) || (0u == uartEnableMask))
        {
        }
    #endif /* (SPI_Card_CY_SCBIP_V1) */

        /* Set default HSIOM to GPIO and Drive Mode to Analog Hi-Z */
        for (i = 0u; i < SPI_Card_SCB_PINS_NUMBER; i++)
        {
            pinsDm[i] = SPI_Card_PIN_DM_ALG_HIZ;
        }

        if ((SPI_Card_SCB_MODE_I2C   == mode) ||
            (SPI_Card_SCB_MODE_EZI2C == mode))
        {
        #if (SPI_Card_RX_SCL_MOSI_PIN)
            hsiomSel[SPI_Card_RX_SCL_MOSI_PIN_INDEX] = SPI_Card_RX_SCL_MOSI_HSIOM_SEL_I2C;
            pinsDm  [SPI_Card_RX_SCL_MOSI_PIN_INDEX] = SPI_Card_PIN_DM_OD_LO;
        #elif (SPI_Card_RX_WAKE_SCL_MOSI_PIN)
            hsiomSel[SPI_Card_RX_WAKE_SCL_MOSI_PIN_INDEX] = SPI_Card_RX_WAKE_SCL_MOSI_HSIOM_SEL_I2C;
            pinsDm  [SPI_Card_RX_WAKE_SCL_MOSI_PIN_INDEX] = SPI_Card_PIN_DM_OD_LO;
        #else
        #endif /* (SPI_Card_RX_SCL_MOSI_PIN) */
        
        #if (SPI_Card_TX_SDA_MISO_PIN)
            hsiomSel[SPI_Card_TX_SDA_MISO_PIN_INDEX] = SPI_Card_TX_SDA_MISO_HSIOM_SEL_I2C;
            pinsDm  [SPI_Card_TX_SDA_MISO_PIN_INDEX] = SPI_Card_PIN_DM_OD_LO;
        #endif /* (SPI_Card_TX_SDA_MISO_PIN) */
        }
    #if (!SPI_Card_CY_SCBIP_V1)
        else if (SPI_Card_SCB_MODE_SPI == mode)
        {
        #if (SPI_Card_RX_SCL_MOSI_PIN)
            hsiomSel[SPI_Card_RX_SCL_MOSI_PIN_INDEX] = SPI_Card_RX_SCL_MOSI_HSIOM_SEL_SPI;
        #elif (SPI_Card_RX_WAKE_SCL_MOSI_PIN)
            hsiomSel[SPI_Card_RX_WAKE_SCL_MOSI_PIN_INDEX] = SPI_Card_RX_WAKE_SCL_MOSI_HSIOM_SEL_SPI;
        #else
        #endif /* (SPI_Card_RX_SCL_MOSI_PIN) */
        
        #if (SPI_Card_TX_SDA_MISO_PIN)
            hsiomSel[SPI_Card_TX_SDA_MISO_PIN_INDEX] = SPI_Card_TX_SDA_MISO_HSIOM_SEL_SPI;
        #endif /* (SPI_Card_TX_SDA_MISO_PIN) */
        
        #if (SPI_Card_CTS_SCLK_PIN)
            hsiomSel[SPI_Card_CTS_SCLK_PIN_INDEX] = SPI_Card_CTS_SCLK_HSIOM_SEL_SPI;
        #endif /* (SPI_Card_CTS_SCLK_PIN) */

            if (SPI_Card_SPI_SLAVE == subMode)
            {
                /* Slave */
                pinsDm[SPI_Card_RX_SCL_MOSI_PIN_INDEX] = SPI_Card_PIN_DM_DIG_HIZ;
                pinsDm[SPI_Card_TX_SDA_MISO_PIN_INDEX] = SPI_Card_PIN_DM_STRONG;
                pinsDm[SPI_Card_CTS_SCLK_PIN_INDEX] = SPI_Card_PIN_DM_DIG_HIZ;

            #if (SPI_Card_RTS_SS0_PIN)
                /* Only SS0 is valid choice for Slave */
                hsiomSel[SPI_Card_RTS_SS0_PIN_INDEX] = SPI_Card_RTS_SS0_HSIOM_SEL_SPI;
                pinsDm  [SPI_Card_RTS_SS0_PIN_INDEX] = SPI_Card_PIN_DM_DIG_HIZ;
            #endif /* (SPI_Card_RTS_SS0_PIN) */

            #if (SPI_Card_TX_SDA_MISO_PIN)
                /* Disable input buffer */
                 pinsInBuf |= SPI_Card_TX_SDA_MISO_PIN_MASK;
            #endif /* (SPI_Card_TX_SDA_MISO_PIN) */
            }
            else 
            {
                /* (Master) */
                pinsDm[SPI_Card_RX_SCL_MOSI_PIN_INDEX] = SPI_Card_PIN_DM_STRONG;
                pinsDm[SPI_Card_TX_SDA_MISO_PIN_INDEX] = SPI_Card_PIN_DM_DIG_HIZ;
                pinsDm[SPI_Card_CTS_SCLK_PIN_INDEX] = SPI_Card_PIN_DM_STRONG;

            #if (SPI_Card_RTS_SS0_PIN)
                hsiomSel [SPI_Card_RTS_SS0_PIN_INDEX] = SPI_Card_RTS_SS0_HSIOM_SEL_SPI;
                pinsDm   [SPI_Card_RTS_SS0_PIN_INDEX] = SPI_Card_PIN_DM_STRONG;
                pinsInBuf |= SPI_Card_RTS_SS0_PIN_MASK;
            #endif /* (SPI_Card_RTS_SS0_PIN) */

            #if (SPI_Card_SS1_PIN)
                hsiomSel [SPI_Card_SS1_PIN_INDEX] = SPI_Card_SS1_HSIOM_SEL_SPI;
                pinsDm   [SPI_Card_SS1_PIN_INDEX] = SPI_Card_PIN_DM_STRONG;
                pinsInBuf |= SPI_Card_SS1_PIN_MASK;
            #endif /* (SPI_Card_SS1_PIN) */

            #if (SPI_Card_SS2_PIN)
                hsiomSel [SPI_Card_SS2_PIN_INDEX] = SPI_Card_SS2_HSIOM_SEL_SPI;
                pinsDm   [SPI_Card_SS2_PIN_INDEX] = SPI_Card_PIN_DM_STRONG;
                pinsInBuf |= SPI_Card_SS2_PIN_MASK;
            #endif /* (SPI_Card_SS2_PIN) */

            #if (SPI_Card_SS3_PIN)
                hsiomSel [SPI_Card_SS3_PIN_INDEX] = SPI_Card_SS3_HSIOM_SEL_SPI;
                pinsDm   [SPI_Card_SS3_PIN_INDEX] = SPI_Card_PIN_DM_STRONG;
                pinsInBuf |= SPI_Card_SS3_PIN_MASK;
            #endif /* (SPI_Card_SS3_PIN) */

                /* Disable input buffers */
            #if (SPI_Card_RX_SCL_MOSI_PIN)
                pinsInBuf |= SPI_Card_RX_SCL_MOSI_PIN_MASK;
            #elif (SPI_Card_RX_WAKE_SCL_MOSI_PIN)
                pinsInBuf |= SPI_Card_RX_WAKE_SCL_MOSI_PIN_MASK;
            #else
            #endif /* (SPI_Card_RX_SCL_MOSI_PIN) */

            #if (SPI_Card_CTS_SCLK_PIN)
                pinsInBuf |= SPI_Card_CTS_SCLK_PIN_MASK;
            #endif /* (SPI_Card_CTS_SCLK_PIN) */
            }
        }
        else /* UART */
        {
            if (SPI_Card_UART_MODE_SMARTCARD == subMode)
            {
                /* SmartCard */
            #if (SPI_Card_TX_SDA_MISO_PIN)
                hsiomSel[SPI_Card_TX_SDA_MISO_PIN_INDEX] = SPI_Card_TX_SDA_MISO_HSIOM_SEL_UART;
                pinsDm  [SPI_Card_TX_SDA_MISO_PIN_INDEX] = SPI_Card_PIN_DM_OD_LO;
            #endif /* (SPI_Card_TX_SDA_MISO_PIN) */
            }
            else /* Standard or IrDA */
            {
                if (0u != (SPI_Card_UART_RX_PIN_ENABLE & uartEnableMask))
                {
                #if (SPI_Card_RX_SCL_MOSI_PIN)
                    hsiomSel[SPI_Card_RX_SCL_MOSI_PIN_INDEX] = SPI_Card_RX_SCL_MOSI_HSIOM_SEL_UART;
                    pinsDm  [SPI_Card_RX_SCL_MOSI_PIN_INDEX] = SPI_Card_PIN_DM_DIG_HIZ;
                #elif (SPI_Card_RX_WAKE_SCL_MOSI_PIN)
                    hsiomSel[SPI_Card_RX_WAKE_SCL_MOSI_PIN_INDEX] = SPI_Card_RX_WAKE_SCL_MOSI_HSIOM_SEL_UART;
                    pinsDm  [SPI_Card_RX_WAKE_SCL_MOSI_PIN_INDEX] = SPI_Card_PIN_DM_DIG_HIZ;
                #else
                #endif /* (SPI_Card_RX_SCL_MOSI_PIN) */
                }

                if (0u != (SPI_Card_UART_TX_PIN_ENABLE & uartEnableMask))
                {
                #if (SPI_Card_TX_SDA_MISO_PIN)
                    hsiomSel[SPI_Card_TX_SDA_MISO_PIN_INDEX] = SPI_Card_TX_SDA_MISO_HSIOM_SEL_UART;
                    pinsDm  [SPI_Card_TX_SDA_MISO_PIN_INDEX] = SPI_Card_PIN_DM_STRONG;
                    
                    /* Disable input buffer */
                    pinsInBuf |= SPI_Card_TX_SDA_MISO_PIN_MASK;
                #endif /* (SPI_Card_TX_SDA_MISO_PIN) */
                }

            #if !(SPI_Card_CY_SCBIP_V0 || SPI_Card_CY_SCBIP_V1)
                if (SPI_Card_UART_MODE_STD == subMode)
                {
                    if (0u != (SPI_Card_UART_CTS_PIN_ENABLE & uartEnableMask))
                    {
                        /* CTS input is multiplexed with SCLK */
                    #if (SPI_Card_CTS_SCLK_PIN)
                        hsiomSel[SPI_Card_CTS_SCLK_PIN_INDEX] = SPI_Card_CTS_SCLK_HSIOM_SEL_UART;
                        pinsDm  [SPI_Card_CTS_SCLK_PIN_INDEX] = SPI_Card_PIN_DM_DIG_HIZ;
                    #endif /* (SPI_Card_CTS_SCLK_PIN) */
                    }

                    if (0u != (SPI_Card_UART_RTS_PIN_ENABLE & uartEnableMask))
                    {
                        /* RTS output is multiplexed with SS0 */
                    #if (SPI_Card_RTS_SS0_PIN)
                        hsiomSel[SPI_Card_RTS_SS0_PIN_INDEX] = SPI_Card_RTS_SS0_HSIOM_SEL_UART;
                        pinsDm  [SPI_Card_RTS_SS0_PIN_INDEX] = SPI_Card_PIN_DM_STRONG;
                        
                        /* Disable input buffer */
                        pinsInBuf |= SPI_Card_RTS_SS0_PIN_MASK;
                    #endif /* (SPI_Card_RTS_SS0_PIN) */
                    }
                }
            #endif /* !(SPI_Card_CY_SCBIP_V0 || SPI_Card_CY_SCBIP_V1) */
            }
        }
    #endif /* (!SPI_Card_CY_SCBIP_V1) */

    /* Configure pins: set HSIOM, DM and InputBufEnable */
    /* Note: the DR register settings do not effect the pin output if HSIOM is other than GPIO */

    #if (SPI_Card_RX_SCL_MOSI_PIN)
        SPI_Card_SET_HSIOM_SEL(SPI_Card_RX_SCL_MOSI_HSIOM_REG,
                                       SPI_Card_RX_SCL_MOSI_HSIOM_MASK,
                                       SPI_Card_RX_SCL_MOSI_HSIOM_POS,
                                        hsiomSel[SPI_Card_RX_SCL_MOSI_PIN_INDEX]);

        SPI_Card_uart_rx_i2c_scl_spi_mosi_SetDriveMode((uint8) pinsDm[SPI_Card_RX_SCL_MOSI_PIN_INDEX]);

        #if (!SPI_Card_CY_SCBIP_V1)
            SPI_Card_SET_INP_DIS(SPI_Card_uart_rx_i2c_scl_spi_mosi_INP_DIS,
                                         SPI_Card_uart_rx_i2c_scl_spi_mosi_MASK,
                                         (0u != (pinsInBuf & SPI_Card_RX_SCL_MOSI_PIN_MASK)));
        #endif /* (!SPI_Card_CY_SCBIP_V1) */
    
    #elif (SPI_Card_RX_WAKE_SCL_MOSI_PIN)
        SPI_Card_SET_HSIOM_SEL(SPI_Card_RX_WAKE_SCL_MOSI_HSIOM_REG,
                                       SPI_Card_RX_WAKE_SCL_MOSI_HSIOM_MASK,
                                       SPI_Card_RX_WAKE_SCL_MOSI_HSIOM_POS,
                                       hsiomSel[SPI_Card_RX_WAKE_SCL_MOSI_PIN_INDEX]);

        SPI_Card_uart_rx_wake_i2c_scl_spi_mosi_SetDriveMode((uint8)
                                                               pinsDm[SPI_Card_RX_WAKE_SCL_MOSI_PIN_INDEX]);

        SPI_Card_SET_INP_DIS(SPI_Card_uart_rx_wake_i2c_scl_spi_mosi_INP_DIS,
                                     SPI_Card_uart_rx_wake_i2c_scl_spi_mosi_MASK,
                                     (0u != (pinsInBuf & SPI_Card_RX_WAKE_SCL_MOSI_PIN_MASK)));

         /* Set interrupt on falling edge */
        SPI_Card_SET_INCFG_TYPE(SPI_Card_RX_WAKE_SCL_MOSI_INTCFG_REG,
                                        SPI_Card_RX_WAKE_SCL_MOSI_INTCFG_TYPE_MASK,
                                        SPI_Card_RX_WAKE_SCL_MOSI_INTCFG_TYPE_POS,
                                        SPI_Card_INTCFG_TYPE_FALLING_EDGE);
    #else
    #endif /* (SPI_Card_RX_WAKE_SCL_MOSI_PIN) */

    #if (SPI_Card_TX_SDA_MISO_PIN)
        SPI_Card_SET_HSIOM_SEL(SPI_Card_TX_SDA_MISO_HSIOM_REG,
                                       SPI_Card_TX_SDA_MISO_HSIOM_MASK,
                                       SPI_Card_TX_SDA_MISO_HSIOM_POS,
                                        hsiomSel[SPI_Card_TX_SDA_MISO_PIN_INDEX]);

        SPI_Card_uart_tx_i2c_sda_spi_miso_SetDriveMode((uint8) pinsDm[SPI_Card_TX_SDA_MISO_PIN_INDEX]);

    #if (!SPI_Card_CY_SCBIP_V1)
        SPI_Card_SET_INP_DIS(SPI_Card_uart_tx_i2c_sda_spi_miso_INP_DIS,
                                     SPI_Card_uart_tx_i2c_sda_spi_miso_MASK,
                                    (0u != (pinsInBuf & SPI_Card_TX_SDA_MISO_PIN_MASK)));
    #endif /* (!SPI_Card_CY_SCBIP_V1) */
    #endif /* (SPI_Card_RX_SCL_MOSI_PIN) */

    #if (SPI_Card_CTS_SCLK_PIN)
        SPI_Card_SET_HSIOM_SEL(SPI_Card_CTS_SCLK_HSIOM_REG,
                                       SPI_Card_CTS_SCLK_HSIOM_MASK,
                                       SPI_Card_CTS_SCLK_HSIOM_POS,
                                       hsiomSel[SPI_Card_CTS_SCLK_PIN_INDEX]);

        SPI_Card_uart_cts_spi_sclk_SetDriveMode((uint8) pinsDm[SPI_Card_CTS_SCLK_PIN_INDEX]);

        SPI_Card_SET_INP_DIS(SPI_Card_uart_cts_spi_sclk_INP_DIS,
                                     SPI_Card_uart_cts_spi_sclk_MASK,
                                     (0u != (pinsInBuf & SPI_Card_CTS_SCLK_PIN_MASK)));
    #endif /* (SPI_Card_CTS_SCLK_PIN) */

    #if (SPI_Card_RTS_SS0_PIN)
        SPI_Card_SET_HSIOM_SEL(SPI_Card_RTS_SS0_HSIOM_REG,
                                       SPI_Card_RTS_SS0_HSIOM_MASK,
                                       SPI_Card_RTS_SS0_HSIOM_POS,
                                       hsiomSel[SPI_Card_RTS_SS0_PIN_INDEX]);

        SPI_Card_uart_rts_spi_ss0_SetDriveMode((uint8) pinsDm[SPI_Card_RTS_SS0_PIN_INDEX]);

        SPI_Card_SET_INP_DIS(SPI_Card_uart_rts_spi_ss0_INP_DIS,
                                     SPI_Card_uart_rts_spi_ss0_MASK,
                                     (0u != (pinsInBuf & SPI_Card_RTS_SS0_PIN_MASK)));
    #endif /* (SPI_Card_RTS_SS0_PIN) */

    #if (SPI_Card_SS1_PIN)
        SPI_Card_SET_HSIOM_SEL(SPI_Card_SS1_HSIOM_REG,
                                       SPI_Card_SS1_HSIOM_MASK,
                                       SPI_Card_SS1_HSIOM_POS,
                                       hsiomSel[SPI_Card_SS1_PIN_INDEX]);

        SPI_Card_spi_ss1_SetDriveMode((uint8) pinsDm[SPI_Card_SS1_PIN_INDEX]);

        SPI_Card_SET_INP_DIS(SPI_Card_spi_ss1_INP_DIS,
                                     SPI_Card_spi_ss1_MASK,
                                     (0u != (pinsInBuf & SPI_Card_SS1_PIN_MASK)));
    #endif /* (SPI_Card_SS1_PIN) */

    #if (SPI_Card_SS2_PIN)
        SPI_Card_SET_HSIOM_SEL(SPI_Card_SS2_HSIOM_REG,
                                       SPI_Card_SS2_HSIOM_MASK,
                                       SPI_Card_SS2_HSIOM_POS,
                                       hsiomSel[SPI_Card_SS2_PIN_INDEX]);

        SPI_Card_spi_ss2_SetDriveMode((uint8) pinsDm[SPI_Card_SS2_PIN_INDEX]);

        SPI_Card_SET_INP_DIS(SPI_Card_spi_ss2_INP_DIS,
                                     SPI_Card_spi_ss2_MASK,
                                     (0u != (pinsInBuf & SPI_Card_SS2_PIN_MASK)));
    #endif /* (SPI_Card_SS2_PIN) */

    #if (SPI_Card_SS3_PIN)
        SPI_Card_SET_HSIOM_SEL(SPI_Card_SS3_HSIOM_REG,
                                       SPI_Card_SS3_HSIOM_MASK,
                                       SPI_Card_SS3_HSIOM_POS,
                                       hsiomSel[SPI_Card_SS3_PIN_INDEX]);

        SPI_Card_spi_ss3_SetDriveMode((uint8) pinsDm[SPI_Card_SS3_PIN_INDEX]);

        SPI_Card_SET_INP_DIS(SPI_Card_spi_ss3_INP_DIS,
                                     SPI_Card_spi_ss3_MASK,
                                     (0u != (pinsInBuf & SPI_Card_SS3_PIN_MASK)));
    #endif /* (SPI_Card_SS3_PIN) */
    }

#endif /* (SPI_Card_SCB_MODE_UNCONFIG_CONST_CFG) */


#if (SPI_Card_CY_SCBIP_V0 || SPI_Card_CY_SCBIP_V1)
    /*******************************************************************************
    * Function Name: SPI_Card_I2CSlaveNackGeneration
    ****************************************************************************//**
    *
    *  Sets command to generate NACK to the address or data.
    *
    *******************************************************************************/
    void SPI_Card_I2CSlaveNackGeneration(void)
    {
        /* Check for EC_AM toggle condition: EC_AM and clock stretching for address are enabled */
        if ((0u != (SPI_Card_CTRL_REG & SPI_Card_CTRL_EC_AM_MODE)) &&
            (0u == (SPI_Card_I2C_CTRL_REG & SPI_Card_I2C_CTRL_S_NOT_READY_ADDR_NACK)))
        {
            /* Toggle EC_AM before NACK generation */
            SPI_Card_CTRL_REG &= ~SPI_Card_CTRL_EC_AM_MODE;
            SPI_Card_CTRL_REG |=  SPI_Card_CTRL_EC_AM_MODE;
        }

        SPI_Card_I2C_SLAVE_CMD_REG = SPI_Card_I2C_SLAVE_CMD_S_NACK;
    }
#endif /* (SPI_Card_CY_SCBIP_V0 || SPI_Card_CY_SCBIP_V1) */


/* [] END OF FILE */
