/*
  =========================================
  Created by : Dwi Wahyudi
  Email : dwiwahyudi1996@gmail.com
  From : Indonesian
  Reference from : Davide Marzorati (https://github.com/dado93)

  copyright Dwi Wahyudi, Cilegon, 2020
  =========================================
*/

/*
 * @brief Function definitions for MPU9250 I2C communication.
 *
 * This file contains the definitions of the functions that can be used
 * to interface with the MPU9250 through the I2C protocol.
 *
 * @date January 19, 2019
 * @author Davide Marzorati
 */
#include "project.h" /* moto */
#include "i2c.h"
#include "mpu9250.h"
#include <string.h>
#include <stdio.h>
#include <ctype.h>

/**
* @brief   0 bit useful when sending a start command. 
*    
* I2C Write operations require to specifiy a 0 when generating a start
* command. This define helps the programmer and reader to understand it better.
*/
#ifndef MPU9250_WRITE
    #define MPU9250_WRITE 0
#endif

/**
* @brief   1 bit useful when sending a start command 
*    
* I2C Read operations require to specifiy a 1 when generating a start
* command. This define helps the programmer and reader to understand it better.
*/
#ifndef MPU9250_READ
    #define MPU9250_READ  1
#endif

/**
* @brief   0 bit for response without acknowledgment.
*    
*/
#ifndef MPU9250_NACK
    #define MPU9250_NACK 0
#endif

/**
* @brief   1 bit for response with acknowledgment.
*    
*/
#ifndef MPU9250_ACK
    #define MPU9250_ACK  1
#endif

uint8_t I2C_MPU9250_Read(uint8_t address, uint8_t reg) {
    /*
        Standard I2C single byte read protocol
            - Send start signal requesting write operation
            - Write byte with register address
            - Send stop signal
            - Send start signal requesting read operation
            - Read byte without acknowledgement
            - Send stop
    */
    
    I2C_MPU9250_I2CMasterSendStart(address,MPU9250_WRITE,I2C_TimeOut);
    I2C_MPU9250_I2CMasterWriteByte(reg, I2C_TimeOut);
	I2C_MPU9250_I2CMasterSendStop(I2C_TimeOut);
	I2C_MPU9250_I2CMasterSendStart(address,MPU9250_READ,I2C_TimeOut);
    uint8_t data = I2C_MPU9250_I2CMasterReadByte(address,MPU9250_NACK,I2C_TimeOut); /* moto */
    I2C_MPU9250_I2CMasterSendStop(I2C_TimeOut); /* moto */
	return data;
}

void I2C_MPU9250_ReadMulti(uint8_t address, uint8_t reg, uint8_t* data, uint16_t count) {
	/*
        Standard I2C multi bytes read protocol
            - Send start signal requesting write operation
            - Write byte with first register address
            - Send restart signal
            - While loop with read byte and acknowledgement
            - Last byte read without acknowledgement
            - Send stop
    */
    I2C_MPU9250_I2CMasterSendStart(address,MPU9250_WRITE,I2C_TimeOut);
    I2C_MPU9250_I2CMasterWriteByte(reg, I2C_TimeOut); /* moto */
    I2C_MPU9250_I2CMasterSendRestart(address,MPU9250_READ, I2C_TimeOut);
	while (count--) {
		if (!count) {
			/* Last byte */
			I2C_MPU9250_I2CMasterReadByte(MPU9250_NACK, data++, I2C_TimeOut); /* moto */
		} else {
			I2C_MPU9250_I2CMasterReadByte(MPU9250_ACK, data++, I2C_TimeOut); /* moto */
		}
	}
    I2C_MPU9250_I2CMasterSendStop(I2C_TimeOut); /* moto */
}

uint8_t I2C_MPU9250_ReadNoRegister(uint8_t address) {
    /*
        Standard I2C no register read protocol
            - Send start signal requesting read operation
            - Read without acknowledgement
            - Send stop
    */
	uint8_t data;
    I2C_MPU9250_I2CMasterSendStart(address,MPU9250_READ, I2C_TimeOut); /* moto */
	/* Also stop condition happens */
	I2C_MPU9250_I2CMasterReadByte(MPU9250_NACK, &data, I2C_TimeOut); /* moto */
    I2C_MPU9250_I2CMasterSendStop(I2C_TimeOut); /* moto */
	return data;
}

void I2C_MPU9250_ReadMultiNoRegister(uint8_t address, uint8_t* data, uint16_t count) {
	/*
        Standard I2C multi bytes no registers read protocol
            - Send start signal requesting read operation
            - While loop with read byte and acknowledgement
            - Last byte read without acknowledgement
            - Send stop
    */
    I2C_MPU9250_I2CMasterSendStart(address, MPU9250_READ, I2C_TimeOut); /* moto */
	while (count--) {
		if (!count) {
			/* Last byte */
			I2C_MPU9250_I2CMasterReadByte(MPU9250_NACK, data, I2C_TimeOut); /* moto */
		} else {
			I2C_MPU9250_I2CMasterReadByte(MPU9250_ACK, data, I2C_TimeOut);  /* moto */
		}
	}
    I2C_MPU9250_I2CMasterSendStop(I2C_TimeOut); /* moto */
}

void I2C_MPU9250_Write(uint8_t address, uint8_t reg, uint8_t data) {
    /*
        Standard I2C single byte write protocol
            - Send start signal requesting write operation
            - Write register byte
            - Write data byte
            - Send stop
    */
    I2C_MPU9250_I2CMasterSendStart(address, MPU9250_WRITE, I2C_TimeOut); /* moto */
    I2C_MPU9250_I2CMasterWriteByte(reg, I2C_TimeOut); /* moto */
	I2C_MPU9250_I2CMasterWriteByte(data, I2C_TimeOut); /* moto */
    I2C_MPU9250_I2CMasterSendStop(I2C_TimeOut); /* moto */
}

void I2C_MPU9250_WriteMulti(uint8_t address, uint8_t reg, uint8_t* data, uint16_t count) {
	/*
        Standard I2C multi bytes write protocol
            - Send start signal requesting write operation
            - While loop with all data to be written
            - Send stop
    */
    I2C_MPU9250_I2CMasterSendStart(address,MPU9250_WRITE,I2C_TimeOut);
    I2C_MPU9250_I2CMasterWriteByte(reg, I2C_TimeOut); /* moto */
	while (count--) {
        I2C_MPU9250_I2CMasterWriteByte(*data++, I2C_TimeOut); /* moto */
	}
    I2C_MPU9250_I2CMasterSendStop(I2C_TimeOut); /* moto */
}

void I2C_MPU9250_WriteNoRegister(uint8_t address, uint8_t data) {
    /*
        Standard I2C single byte no register write protocol
            - Send start signal requesting write operation
            - Write byte
            - Send stop
    */
    I2C_MPU9250_I2CMasterSendStart(address, MPU9250_WRITE, I2C_TimeOut); /* moto */
	I2C_MPU9250_I2CMasterWriteByte(data, I2C_TimeOut); /* moto */
	I2C_MPU9250_I2CMasterSendStop(I2C_TimeOut); /* moto */
}

void I2C_MPU9250_WriteMultiNoRegister(uint8_t address, uint8_t* data, uint16_t count) {
	/*
        Standard I2C multi byte no registers write protocol
            - Send start signal requesting write operation
            - Write all bytes
            - Send stop
    */
    I2C_MPU9250_I2CMasterSendStart(address, MPU9250_WRITE, I2C_TimeOut); /* moto */
	while (count--) {
        I2C_MPU9250_I2CMasterWriteByte(*data++, I2C_TimeOut); /* moto */
	}
	I2C_MPU9250_I2CMasterSendStop(I2C_TimeOut); /* moto */
}
/* [] END OF FILE */

/* [] END OF FILE */
