/*
  =========================================
  Created by : Dwi Wahyudi
  Email : dwiwahyudi1996@gmail.com
  From : Indonesian
  Reference from : Davide Marzorati (https://github.com/dado93)

  copyright Dwi Wahyudi, Cilegon, 2020
  =========================================
*/
#include <cytypes.h>
#include <I2C_MPU9250.h>

#define MPU9250_ADDRESS 0x68       // MPU9250 address when ADO = 1
#define AK8963_ADDRESS 0x0C        // Address of AK8963 (MPU9250) magnetometer

#define MPU9250_SELF_TEST_X_GYRO 0x00
#define MPU9250_SELF_TEST_Y_GYRO 0x01
#define MPU9250_SELF_TEST_Z_GYRO 0x02

#define MPU9250_SELF_TEST_X_ACCEL 0x0D
#define MPU9250_SELF_TEST_Y_ACCEL 0x0E
#define MPU9250_SELF_TEST_Z_ACCEL 0x0F

#define MPU9250_SELF_TEST_A      0x10

#define MPU9250_XG_OFFSET_H      0x13  // User-defined trim values for gyroscope
#define MPU9250_XG_OFFSET_L      0x14
#define MPU9250_YG_OFFSET_H      0x15
#define MPU9250_YG_OFFSET_L      0x16
#define MPU9250_ZG_OFFSET_H      0x17
#define MPU9250_ZG_OFFSET_L      0x18
#define MPU9250_SMPLRT_DIV       0x19
#define MPU9250_CONFIG           0x1A
#define MPU9250_GYRO_CONFIG      0x1B
#define MPU9250_ACCEL_CONFIG     0x1C
#define MPU9250_ACCEL_CONFIG2    0x1D
#define MPU9250_LP_ACCEL_ODR     0x1E
#define MPU9250_WOM_THR          0x1F

#define MPU9250_MOT_DUR          0x20  // Duration counter threshold for motion interrupt generation, 1 kHz rate, LSB = 1 ms
#define MPU9250_ZMOT_THR         0x21  // Zero-motion detection threshold bits [7:0]
#define MPU9250_ZRMOT_DUR        0x22  // Duration counter threshold for zero motion interrupt generation, 16 Hz rate, LSB = 64 ms

#define MPU9250_FIFO_EN          0x23
#define MPU9250_I2C_MST_CTRL     0x24
#define MPU9250_I2C_SLV0_ADDR    0x25
#define MPU9250_I2C_SLV0_REG     0x26
#define MPU9250_I2C_SLV0_CTRL    0x27
#define MPU9250_I2C_SLV1_ADDR    0x28
#define MPU9250_I2C_SLV1_REG     0x29
#define MPU9250_I2C_SLV1_CTRL    0x2A
#define MPU9250_I2C_SLV2_ADDR    0x2B
#define MPU9250_I2C_SLV2_REG     0x2C
#define MPU9250_I2C_SLV2_CTRL    0x2D
#define MPU9250_I2C_SLV3_ADDR    0x2E
#define MPU9250_I2C_SLV3_REG     0x2F
#define MPU9250_I2C_SLV3_CTRL    0x30
#define MPU9250_I2C_SLV4_ADDR    0x31
#define MPU9250_I2C_SLV4_REG     0x32
#define MPU9250_I2C_SLV4_DO      0x33
#define MPU9250_I2C_SLV4_CTRL    0x34
#define MPU9250_I2C_SLV4_DI      0x35
#define MPU9250_I2C_MST_STATUS   0x36
#define MPU9250_INT_PIN_CFG      0x37
#define MPU9250_INT_ENABLE       0x38
#define MPU9250_DMP_INT_STATUS   0x39  // Check DMP interrupt
#define MPU9250_INT_STATUS       0x3A
#define MPU9250_ACCEL_XOUT_H     0x3B
#define MPU9250_ACCEL_XOUT_L     0x3C
#define MPU9250_ACCEL_YOUT_H     0x3D
#define MPU9250_ACCEL_YOUT_L     0x3E
#define MPU9250_ACCEL_ZOUT_H     0x3F
#define MPU9250_ACCEL_ZOUT_L     0x40
#define MPU9250_TEMP_OUT_H       0x41
#define MPU9250_TEMP_OUT_L       0x42
#define MPU9250_GYRO_XOUT_H      0x43
#define MPU9250_GYRO_XOUT_L      0x44
#define MPU9250_GYRO_YOUT_H      0x45
#define MPU9250_GYRO_YOUT_L      0x46
#define MPU9250_GYRO_ZOUT_H      0x47
#define MPU9250_GYRO_ZOUT_L      0x48
#define MPU9250_EXT_SENS_DATA_00 0x49
#define MPU9250_EXT_SENS_DATA_01 0x4A
#define MPU9250_EXT_SENS_DATA_02 0x4B
#define MPU9250_EXT_SENS_DATA_03 0x4C
#define MPU9250_EXT_SENS_DATA_04 0x4D
#define MPU9250_EXT_SENS_DATA_05 0x4E
#define MPU9250_EXT_SENS_DATA_06 0x4F
#define MPU9250_EXT_SENS_DATA_07 0x50
#define MPU9250_EXT_SENS_DATA_08 0x51
#define MPU9250_EXT_SENS_DATA_09 0x52
#define MPU9250_EXT_SENS_DATA_10 0x53
#define MPU9250_EXT_SENS_DATA_11 0x54
#define MPU9250_EXT_SENS_DATA_12 0x55
#define MPU9250_EXT_SENS_DATA_13 0x56
#define MPU9250_EXT_SENS_DATA_14 0x57
#define MPU9250_EXT_SENS_DATA_15 0x58
#define MPU9250_EXT_SENS_DATA_16 0x59
#define MPU9250_EXT_SENS_DATA_17 0x5A
#define MPU9250_EXT_SENS_DATA_18 0x5B
#define MPU9250_EXT_SENS_DATA_19 0x5C
#define MPU9250_EXT_SENS_DATA_20 0x5D
#define MPU9250_EXT_SENS_DATA_21 0x5E
#define MPU9250_EXT_SENS_DATA_22 0x5F
#define MPU9250_EXT_SENS_DATA_23 0x60
#define MPU9250_MOT_DETECT_STATUS 0x61
#define MPU9250_I2C_SLV0_DO      0x63
#define MPU9250_I2C_SLV1_DO      0x64
#define MPU9250_I2C_SLV2_DO      0x65
#define MPU9250_I2C_SLV3_DO      0x66
#define MPU9250_I2C_MST_DELAY_CTRL 0x67
#define MPU9250_SIGNAL_PATH_RESET  0x68
#define MPU9250_MOT_DETECT_CTRL  0x69
#define MPU9250_USER_CTRL        0x6A  // Bit 7 enable DMP, bit 3 reset DMP
#define MPU9250_PWR_MGMT_1       0x6B // Device defaults to the SLEEP mode
#define MPU9250_PWR_MGMT_2       0x6C
#define MPU9250_DMP_BANK         0x6D  // Activates a specific bank in the DMP
#define MPU9250_DMP_RW_PNT       0x6E  // Set read/write pointer to a specific start address in specified DMP bank
#define MPU9250_DMP_REG          0x6F  // Register in DMP from which to read or to which to write
//#define MPU9250_DMP_REG_1        0x70
//#define MPU9250_DMP_REG_2        0x71
#define MPU9250_FIFO_COUNTH      0x72
#define MPU9250_FIFO_COUNTL      0x73
#define MPU9250_FIFO_R_W         0x74
#define MPU9250_WHO_AM_I         0x071 //0x75 // Should return 0x71
#define MPU9250_XA_OFFSET_H      0x77
#define MPU9250_XA_OFFSET_L      0x78
#define MPU9250_YA_OFFSET_H      0x7A
#define MPU9250_YA_OFFSET_L      0x7B
#define MPU9250_ZA_OFFSET_H      0x7D
#define MPU9250_ZA_OFFSET_L      0x7E

//Magnetometer Registers
#define AK8963_ADDRESS   0x0C
#define WHO_AM_I_AK8963  0x00 // should return 0x48
#define INFO             0x01
#define AK8963_ST1       0x02  // data ready status bit 0
#define AK8963_XOUT_L    0x03  // data
#define AK8963_XOUT_H    0x04
#define AK8963_YOUT_L    0x05
#define AK8963_YOUT_H    0x06
#define AK8963_ZOUT_L    0x07
#define AK8963_ZOUT_H    0x08
#define AK8963_ST2       0x09  // Data overflow bit 3 and data read error status bit 2
#define AK8963_CNTL      0x0A  // Power down (0000), single-measurement (0001), self-test (1000) and Fuse ROM (1111) modes on bits 3:0
#define AK8963_ASTC      0x0C  // Self test control
#define AK8963_I2CDIS    0x0F  // I2C disable
#define AK8963_ASAX      0x10  // Fuse ROM x-axis sensitivity adjustment value
#define AK8963_ASAY      0x11  // Fuse ROM y-axis sensitivity adjustment value
#define AK8963_ASAZ      0x12  // Fuse ROM z-axis sensitivity adjustment value

//delay & timeout
#define LED_Delay (500u)

#ifndef __MPU9250__H
    #define __MPU9250__H
    
    /* ========= MACROS ========= */
    
    /**
    * @brief I2C address of the AK8963 magnetometer.
    *
    * This is the I2C address of the  AK8963 magnetometer.
    */
    #define AK8963_I2C_ADDRESS 0x0C
    #define AK8963_I2C_ADDRESS_READ  ((AK8963_I2C_ADDRESS<<1) | 1)
    #define AK8963_I2C_ADDRESS_WRITE ((AK8963_I2C_ADDRESS<<1) | 0)
    
    /* ========= TYPE DEFS ========= */
    
    /** 
     * @brief Values of accelerometer full scale range.
    **/
    typedef enum {
        /** ±2g accelerometer full scale range **/
        MPU9250_Acc_FS_2g, 
        /** ±4g accelerometer full scale range **/
        MPU9250_Acc_FS_4g,
        /** ±8g accelerometer full scale range **/
        MPU9250_Acc_FS_8g,
        /** ±16g accelerometer full scale range **/
        MPU9250_Acc_FS_16g
    } MPU9250_Acc_FS; 
    
    /** 
     * @brief Values of gyroscope full scale range.
    **/
    typedef enum {
        /** +250 DPS gyroscope full scale range **/
        MPU9250_Gyro_FS_250,
        /** +500 DPS gyroscope full scale range **/
        MPU9250_Gyro_FS_500,
        /** +1000 DPS gyroscope full scale range **/
        MPU9250_Gyro_FS_1000,
        /** +2000 DPS gyroscope full scale range **/
        MPU9250_Gyro_FS_2000
    } MPU9250_Gyro_FS;
    
    /* ========= FUNCTIONS DECLARATIONS ========= */
    
    /**
    * @brief Start the component.
    *
    * This function starts the I2C master component, if not already started. This check
    * is done on the initVar of the I2C master component.
    */
    void MPU9250_Start(void);
    
    /**
    * @brief Enter sleep mode.
    *
    * This function puts the MPU9250 into sleep mode. See register #MPU9250_PWR_MGMT_1.
    */
    void MPU9250_Sleep(void);
    
    /**
    * @brief Exit sleep mode.
    *
    * This function wakes up the MPU9250, exiting sleep mode. See register #MPU9250_PWR_MGMT_1.
    */
    void MPU9250_WakeUp(void);
    
    /**
    * @brief Check I2C connection with MPU9250.
    *
    * This function checks if the MPU9250 is connected on the I2C bus.
    * @return Connection status:
    *            - 0: Device is not connected
    *            - > 0: Device is connected
    */
    uint8_t MPU9250_IsConnected(void);
    
    /**
    * @brief Read MPU9250 WHO AM I register.
    *
    * This function reads the #MPU9250_WHO_AM_I register of the 
    * MPU9250. The expected value returned by this function is 
    * #MPU9250_WHO_AM_I
    *
    * @return the value of the WHO AM I register of the MPU9250.
    */
    uint8_t MPU9250_ReadWhoAmI(void);
    
    /**
    * @brief Read the magnetometer WHO AM I register.
    *
    */
    uint8_t MPU9250_ReadMagWhoAmI(void);
    
    /**
    * @brief Read accelerometer values.
    *
    * This function reads the accelerometer values on the three
    * axis (x, y, and z). 
    * @param[out] acc: accelerometer values (x, y, and z).
    *
    */
    void MPU9250_ReadAcc(int16_t* acc);
    
    /**
    * @brief Read accelerometer raw values.
    *
    * This function reads the accelerometer values on the three
    * axis (x, y, and z) and returns the raw values. 
    * @param[out] acc: accelerometer values (xH, xL, yH, yL, zH, zL).
    *
    */
    void MPU9250_ReadAccRaw(uint8_t* acc);
    
    /**
    * @brief Read gyroscope values.
    *
    * This function reads the gyroscope values on the three
    * axis (x, y, and z). 
    * @param[out] gyro: gyroscope values (x, y, and z).
    *
    */
    void MPU9250_ReadGyro(int16_t* gyro);
    
    /**
    * @brief Read gyroscope raw values.
    *
    * This function reads the gyroscope values on the three
    * axis (x, y, and z) and returns the raw values. 
    * @param[out] gyro: gyroscope raw values (xH, xL, yH, yL, zH, zL).
    *
    */
    void MPU9250_ReadGyroRaw(uint8_t* gyro);
    
    /**
    * @brief Read magnetometer raw values.
    *
    * This function reads the magnetometer values on the three
    * axis (x, y, and z) and returns the raw values. 
    * @param[out] mag: magnetometer raw values (xH, xL, yH, yL, zH, zL).
    *
    */
    void MPU9250_ReadMag(uint16_t* mag);
    
     /**
    * @brief Read magnetometer raw values.
    *
    * This function reads the magnetometer values on the three
    * axis (x, y, and z) and returns the raw values. 
    * @param[out] mag: magnetometer raw values (xH, xL, yH, yL, zH, zL).
    *
    */
    void MPU9250_ReadMagRaw(uint8_t* mag);
    
    /**
    * @brief Read temperature.
    *
    * This function reads the temperature value.
    * @param[out] temp: temperature value.
    *
    */
    void MPU9250_ReadTemp(int16_t* temp);
    
    
    /**
    * @brief Read accelerometer and gyroscope values.
    *
    * This function reads the accelerometer and gyroscope values on the three
    * axis (x, y, and z). 
    * @param[out] acc: accelerometer values (x, y, and z).
    * @param[out] gyro: gyroscope values (x, y, and z).
    *
    */
    void MPU9250_ReadAccGyro(int16_t* acc, int16_t* gyro);
    
    /**
    * @brief Read accelerometer and gyroscope raw values.
    *
    * This function reads the accelerometer and gyroscope values on the three
    * axis (x, y, and z). 
    * @param[out] data: accelerometer and gyro raw values (xH, xL, yH, yL, zH, zL).
    *
    */
    void MPU9250_ReadAccGyroRaw(uint8_t* data);
    
    /*
    void MPU9250_ReadMag(void);
    */
    
    void MPU9250_ReadSelfTestGyro(uint8_t* self_test_gyro);
    void MPU9250_ReadSelfTestAcc(uint8_t* self_test_acc);
    
    /**
    * @brief Perform self test of accelerometer and gyroscope.
    * 
    * This function performs a self test of the MPU9250. The returned values 
    * represent percent deviation from factory trim values. For a more detailed
    * explanation please check the MPU-9250 Accelerometer, Gyroscope and
    * Compass Self-Test Implementation application.
    */
    void MPU9250_SelfTest(float* deviation);
    
    /**
    * \brief Perform magnetometer self test.
    *
    */
    void MPU9250_SelfTestMag(float deviation);
    
    /**
    * @brief Enable accelerometer.
    *
    * Activate accelerometer updates. See register #MPU9250_PWR_MGMT_2_REG.
    *
    */
    void MPU9250_EnableAcc(void);
    
    /**
    * @brief Enable gyroscope.
    *
    * Activate gyroscope updates. See register #MPU9250_PWR_MGMT_2_REG.
    *
    */
    void MPU9250_EnableGyro(void);
    
    void MPU9250_EnableMag(void);
    
    /**
    * @brief Disable accelerometer.
    *
    * Disable accelerometer updates. See register #MPU9250_PWR_MGMT_2_REG.
    *
    */
    void MPU9250_DisableAcc(void);
    
    /**
    * @brief Disable gyroscope.
    *
    * Activate gyroscope updates. See register #MPU9250_PWR_MGMT_2_REG.
    *
    */
    void MPU9250_DisableGyro(void);
    
    void MPU9250_DisableMag(void);
    
    
    /**
    * @brief Set the accelerometer full scale range.
    *
    * This function sets accelerometer full scale range for the
    * accelerometer. See #MPU9250_Acc_FS and #MPU9250_ACCEL_CONFIG
    *
    * @param fs: the new full scale range value to be used.
    **/
    void MPU9250_SetAccFS(MPU9250_Acc_FS fs);
    
    /**
    * @brief Get the accelerometer full scale range.
    *
    * This function gets accelerometer full scale rang. 
    * See #MPU9250_Acc_FS and #MPU9250_ACCEL_CONFIG
    *
    * @return the accelerometer full scale range.
    **/
    MPU9250_Acc_FS MPU9250_GetAccFS(void);
    
    /**
    * @brief Set the gyroscope full scale range.
    *
    * This function sets the full scale range for the
    * gyroscope. See #MPU9250_Gyro_FS and #MPU9250_GYRO_CONFIG
    *
    * @param fs: the new full scale range value to be used.
    **/
    void MPU9250_SetGyroFS(MPU9250_Gyro_FS fs);
    
    /**
    * @brief Get the gyroscope full scale range.
    *
    * This function gets the gyroscope full scale range.
    * See #MPU9250_Gyro_FS and #MPU9250_GYRO_CONFIG
    *
    * @return the gyroscope full scale range.
    **/
    MPU9250_Gyro_FS MPU9250_GetGyroFS(void);
    
    /**
    * @brief Set sample rate divider.
    *
    * This function sets the divider for the internal sample rate (see also register 
    * #MPU9250_CONFIG_REG) to generate the sample rate that controls data
    * output rate, FIFO sample rate. This register is only effective when 
    * Fchoice = 2'b11 (fchoice_b register bits are 2'b00) and (0 < dlpf_cfg < 7), 
    * such that the averate filter's output is selected. 
    *
    * SAMPLE RATE = Internal_Sample_Rate / ( 1 + SMPLRT_DIV)
    **/
    void MPU9250_SetSampleRateDivider(uint8_t smplrt);
    
    /**
    * @brief Read accelerometer offset values.
    *
    * This function reads the accelerometer offset values on all the axis (x, y, and z).
    * @param[out] acc_offset: array where the 3 offset values will be stored
    **/
    void MPU9250_ReadAccelerometerOffset(int16_t *acc_offset);
    
    /**
    * @brief Enable interrupt on raw sensor data ready.
    *
    * This function eables raw sensor data ready interrupt to propagate to interrupt
    * pin. The timing of the interrupt can vary depending on the setting in register
    * 36 #I2C_MST_CTRL , bit[6] WAIT_FOR_ES
    **/
    void MPU9250_EnableRawDataInterrupt(void);
    
    /**
    * @brief Disable interrupt on raw sensor data ready.
    **/
    void MPU9250_DisableRawDataInterrupt(void);
    
    /**
    * @brief Enable Fsync interrupt to propagate to interrupt pin.
    */
    void MPU9250_EnableFsyncInterrupt(void);
    
    /**
    * @brief Disbale Fsync interrupt to propagate to interrupt pin.
    */
    void MPU9250_DisableFsyncInterrupt(void);
    
    /**
    * @brief Enable interrupt for fifo overflow to propagate to interrupt pin.
    */
    void MPU9250_EnableFifoOverflowInterrupt(void);
    
     /**
    * @brief Disable interrupt for fifo overflow to propagate to interrupt pin.
    */
    void MPU9250_DisableFifoOverflowInterrupt(void);
    
    /**
    * @brief Enable interrupt for wake on motion to propagate to interrupt pin.
    */
    void MPU9250_EnableWomInterrupt(void);
    
    /**
    * @brief Disable interrupt for wake on motion to propagate to interrupt pin.
    */
    void MPU9250_DisableWomInterrupt(void);
    
    /**
    * @brief Read interrupt status register.
    * 
    * This function reads the interrupt status register. For additional information
    * check register #MPU9250_INT_STATUS_REG
    * @return value of the interrupt status register.
    */
    uint8_t MPU9250_ReadInterruptStatus(void);
    
    
    /**
    * @brief Set interrupt pin as active high.
    *
    * This function sets the interrupt logic level as active high.
    */
    void MPU9250_SetInterruptActiveHigh(void);
    
    /**
    * @brief Set interrupt pin as active low.
    *
    * This function sets the interrupt logic low as active high.
    */
    void MPU9250_SetInterruptActiveLow(void);
    
    /**
    * @brief Set interrupt pin as open drain.
    *
    * This function sets the interrupt pin in open drain configuration.
    */
    void MPU9250_SetInterruptOpenDrain(void);
    
    /**
    * @brief Set interrupt pin as push pull.
    *
    * This function sets the interrupt pin in push - pull configuration.
    */
    void MPU9250_SetInterruptPushPull(void);
    
    /**
    * @brief Enable I2C bypass.
    *
    * This function enable the I2C bypass. This way, the I2C master interface pins
    * will go into bypass mode when the I2C master interface is disabled. This pins 
    * will float high due to the internal pull-up if not enabled and the I2C 
    * master interface is disabled.
    */
    void MPU9250_EnableI2CBypass(void);
    
    /**
    * @brief Disable I2C bypass.
    *
    */
    void MPU9250_DisableI2CBypass(void);
    
    /**
    * @brief Held interrupt pin until interrupt status is cleared.
    *
    * This function sets up the interrupt to be held until the interrupt status is cleared.
    */
    void MPU9250_HeldInterruptPin(void);
    
    /**
    * @brief Interrupt pin pulse.
    *
    * This function sets up the interrupt to a 50 us pulse.
    */
    void MPU9250_InterruptPinPulse(void);
    
    /**
    * @brief Clear interrupt on any read operation.
    *
    * This function enables the interrupt to be cleared when any read operation is
    * performed.
    */
    void MPU9250_ClearInterruptAny(void);
    
    /**
    * @brief Clear interrupt only when reading status register.
    *
    * This function enables the interrupt to be cleared only when the status register
    * is read.
    */
    void MPU9250_ClearInterruptStatusReg(void);
    
    /**
    * @brief Enable the 3-axis magnetometer
    *
    * This function enables the built-in 3-axis magnetometer.
    */
    void MPU9250_MAG_Enable(void);
    
    /**
    * @brief Disable the 3-axis magnetometer
    *
    * This function disables the built-in 3-axis magnetometer.
    */
    void MPU9250_MAG_Disable(void);
    
#endif

/* [] END OF FILE */
